@@ -585,10 +585,16 @@ SCAN_TYPE_NONE = 0
 SCAN_TYPE_SCL = 1
 SCAN_TYPE_PML = 2
 
+# scan-styles
+SCAN_STYLE_NONE = 0
+SCAN_STYLE_FLATBED = 1
+SCAN_STYLE_SCROLLFED = 2
+
 # copy-types
 COPY_TYPE_NONE = 0
-COPY_TYPE_DEVICE = 1
+COPY_TYPE_DEVICE = 1 # LaserJet MFP PML
 COPY_TYPE_SCAN_TO_PRINT = 2
+COPY_TYPE_AIO_DEVICE = 3 # Inkjet AiO PML
 
 # 'top_door' (lid)
 TOP_DOOR_NOT_PRESENT = 0
