@@ -246,6 +246,15 @@ public:
 
     void    ResetIOMode (BOOL bDevID, BOOL bStatus);
 
+    int     GetCopyCount ()
+    {
+        return m_iCopyCount;
+    }
+    void    SetCopyCount (int iNumCopies)
+    {
+        m_iCopyCount = iNumCopies;
+    }
+
 private:
 
     SystemServices* pSS;
@@ -329,6 +338,8 @@ private:
 
 #endif
 
+    int m_iCopyCount;
+
 }; //PrintContext
 
 APDK_END_NAMESPACE
