@@ -171,15 +171,20 @@ class PrinterForm(PrinterForm_base):
 
     def InitialUpdate(self):
         if self.init_failed:
-            #log.error("Init failed")
             self.close()
             return        
 
         self.printer_list = []
 
-        self.dev = device.Device(device_uri=self.device_uri, 
-                                 printer_name=self.printer_name, 
-                                 hpssd_sock=self.sock)
+        try:
+            self.dev = device.Device(device_uri=self.device_uri, 
+                                     printer_name=self.printer_name, 
+                                     hpssd_sock=self.sock)
+        except Error, e:
+            log.error("Invalid device URI or printer name.")
+            self.FailureUI("<b>Invalid device URI or printer name.</b><p>Please check the parameters to hp-print and try again.")
+            self.close()
+            return
 
         self.device_uri = self.dev.device_uri
 
@@ -383,13 +388,24 @@ class PrinterForm(PrinterForm_base):
             
             alt_nup = (nup > 1 and t == 'application/postscript' and utils.which('psnup'))
                 
-            if alt_nup:
-                cmd = ' '.join(['psnup', '-%d' % nup, ''.join(['"', p, '"']), '| lp -c -d', self.current_printer])
+            if utils.which('lpr'):
+                if alt_nup:
+                    cmd = ' '.join(['psnup', '-%d' % nup, ''.join(['"', p, '"']), '| lpr -P', self.current_printer])
+                else:
+                    cmd = ' '.join(['lpr -P', self.current_printer])
+                
+                if copies > 1:
+                    cmd = ' '.join([cmd, '-#%d' % copies])
+                
             else:
-                cmd = ' '.join(['lp -c -d', self.current_printer])
+                if alt_nup:
+                    cmd = ' '.join(['psnup', '-%d' % nup, ''.join(['"', p, '"']), '| lp -c -d', self.current_printer])
+                else:
+                    cmd = ' '.join(['lp -c -d', self.current_printer])
+                
+                if copies > 1:
+                    cmd = ' '.join([cmd, '-n%d' % copies])
 
-            if copies > 1:
-                cmd = ' '.join([cmd, '-n%d' % copies])
 
             if not all_pages and len(page_range) > 0:
                 cmd = ' '.join([cmd, '-o page-ranges=%s' % page_range])
